@@ -36,9 +36,10 @@
 #include <QLabel>
 #include <QPushButton>
 #include <QMessageBox>
-#include <QDebug>
 #include <QPageSize>
 
+#include <ksane_debug.h>
+
 #define SCALED_PREVIEW_MAX_SIDE 400
 
 static const int ActiveSelection = 100000;
@@ -172,7 +173,7 @@ void KSaneWidgetPrivate::devListUpdated()
         }
         for (int i = 0; i < list.size(); ++i) {
             const KSaneWidget::DeviceInfo info = list.at(i);
-            qDebug() << info.name;
+            qCDebug(KSANE_LOG) << info.name;
             if (info.name == m_devName) {
                 m_vendor    = info.vendor;
                 m_model     = info.model;
@@ -184,7 +185,7 @@ void KSaneWidgetPrivate::devListUpdated()
 
 void KSaneWidgetPrivate::signalDevListUpdate()
 {
-    emit(q->availableDevices(m_findDevThread->devicesList()));
+    emit q->availableDevices(m_findDevThread->devicesList());
 }
 
 KSaneWidget::ImageFormat KSaneWidgetPrivate::getImgFormat(SANE_Parameters &params)
@@ -328,7 +329,7 @@ float KSaneWidgetPrivate::ratioToDispUnitX(float ratio)
             return mmToDispUnit(result);
         }
     }
-    qDebug() << "Failed to convert scan area to mm";
+    qCDebug(KSANE_LOG) << "Failed to convert scan area to mm";
     return ratio;
 }
 
@@ -352,7 +353,7 @@ float KSaneWidgetPrivate::ratioToDispUnitY(float ratio)
             return mmToDispUnit(result);
         }
     }
-    qDebug() << "Failed to convert scan area to display unit";
+    qCDebug(KSANE_LOG) << "Failed to convert scan area to display unit";
     return ratio;
 }
 
@@ -729,10 +730,7 @@ void KSaneWidgetPrivate::optReload()
 
 void KSaneWidgetPrivate::valReload()
 {
-    int i;
-    QString tmp;
-
-    for (i = 0; i < m_optList.size(); ++i) {
+    for (int i = 0; i < m_optList.size(); ++i) {
         m_optList.at(i)->readValue();
     }
 
@@ -872,7 +870,7 @@ void KSaneWidgetPrivate::updatePreviewSize()
         m_optBrY->getMaxValue(max_y);
     }
     if ((max_x == m_previewWidth) && (max_y == m_previewHeight)) {
-        //qDebug() << "no preview size change";
+        //qCDebug(KSANE_LOG) << "no preview size change";
         return;
     }
 
@@ -898,7 +896,7 @@ void KSaneWidgetPrivate::updatePreviewSize()
 
     // Avoid crash if max_y or max_x == 0
     if (max_x < 0.0001 || max_y < 0.0001) {
-        qWarning() << "Risk for division by 0" << max_x << max_y;
+        qCWarning(KSANE_LOG) << "Risk for division by 0" << max_x << max_y;
         return;
     }
 
@@ -996,7 +994,7 @@ void KSaneWidgetPrivate::startPreviewScan()
                 //check what image size we would get in a scan
                 status = sane_get_parameters(m_saneHandle, &params);
                 if (status != SANE_STATUS_GOOD) {
-                    qDebug() << "sane_get_parameters=" << sane_strstatus(status);
+                    qCDebug(KSANE_LOG) << "sane_get_parameters=" << sane_strstatus(status);
                     previewScanDone();
                     return;
                 }
@@ -1013,7 +1011,7 @@ void KSaneWidgetPrivate::startPreviewScan()
                 // This is a security measure for broken backends
                 m_optRes->getMinValue(dpi);
                 m_optRes->setValue(dpi);
-                qDebug() << "Setting minimum DPI value for a broken back-end";
+                qCDebug(KSANE_LOG) << "Setting minimum DPI value for a broken back-end";
             }
         }
     }
@@ -1054,7 +1052,7 @@ void KSaneWidgetPrivate::previewScanDone()
         sane_close(m_saneHandle);
         m_saneHandle = nullptr;
         clearDeviceOptions();
-        emit(q->scanDone(KSaneWidget::NoError, QStringLiteral("")));
+        emit q->scanDone(KSaneWidget::NoError, QString());
         return;
     }
 
@@ -1080,7 +1078,7 @@ void KSaneWidgetPrivate::previewScanDone()
 
     if ((m_previewThread->saneStatus() != SANE_STATUS_GOOD) &&
             (m_previewThread->saneStatus() != SANE_STATUS_EOF)) {
-        alertUser(KSaneWidget::ErrorGeneral, i18n(sane_strstatus(m_previewThread->saneStatus())));
+        alertUser(KSaneWidget::ErrorGeneral, sane_i18n(sane_strstatus(m_previewThread->saneStatus())));
     } else if (m_autoSelect) {
         m_previewViewer->findSelections();
     }
@@ -1089,7 +1087,7 @@ void KSaneWidgetPrivate::previewScanDone()
     m_scanOngoing = false;
     m_updProgressTmr.stop();
 
-    emit(q->scanDone(KSaneWidget::NoError, QStringLiteral("")));
+    emit q->scanDone(KSaneWidget::NoError, QString());
 
     return;
 }
@@ -1101,6 +1099,7 @@ void KSaneWidgetPrivate::startFinalScan()
     }
     m_scanOngoing = true;
     m_isPreview = false;
+    m_cancelMultiScan = false;
 
     float x1 = 0, y1 = 0, x2 = 0, y2 = 0;
 
@@ -1153,11 +1152,11 @@ void KSaneWidgetPrivate::oneFinalScanDone()
             int bytesPerLine = qMax(getBytesPerLines(params), 1); // ensure no div by 0
             lines = m_scanData.size() / bytesPerLine;
         }
-        emit(q->imageReady(m_scanData,
+        emit q->imageReady(m_scanData,
                            params.pixels_per_line,
                            lines,
                            getBytesPerLines(params),
-                           (int)getImgFormat(params)));
+                           (int)getImgFormat(params));
 
         // now check if we should have automatic ADF batch scanning
         if (m_optSource) {
@@ -1168,23 +1167,27 @@ void KSaneWidgetPrivate::oneFinalScanDone()
                 source.contains(QStringLiteral("ADF")) ||
                 source.contains(QStringLiteral("Duplex"))) {
                 // in batch mode only one area can be scanned per page
-                //qDebug() << "source == " << source;
-                m_updProgressTmr.start();
-                m_scanThread->start();
-                return;
+                //qCDebug(KSANE_LOG) << "source == " << source;
+                if (!m_cancelMultiScan) {
+                    m_updProgressTmr.start();
+                    m_scanThread->start();
+                    m_cancelMultiScan = false;
+                    return;
+                }
+
             }
         }
 
         // Check if we have a "wait for button" batch scanning
         if (m_optWaitForBtn) {
-            qDebug() << m_optWaitForBtn->name();
+            qCDebug(KSANE_LOG) << m_optWaitForBtn->name();
             QString wait;
             m_optWaitForBtn->getValue(wait);
 
-            qDebug() << "wait ==" << wait;
+            qCDebug(KSANE_LOG) << "wait ==" << wait;
             if (wait == QStringLiteral("true")) {
                 // in batch mode only one area can be scanned per page
-                //qDebug() << "source == \"Automatic Document Feeder\"";
+                //qCDebug(KSANE_LOG) << "source == \"Automatic Document Feeder\"";
                 m_updProgressTmr.start();
                 m_scanThread->start();
                 return;
@@ -1194,7 +1197,7 @@ void KSaneWidgetPrivate::oneFinalScanDone()
         // not batch scan, call sane_cancel to be able to change parameters.
         sane_cancel(m_saneHandle);
 
-        //qDebug() << "index=" << m_selIndex << "size=" << m_previewViewer->selListSize();
+        //qCDebug(KSANE_LOG) << "index=" << m_selIndex << "size=" << m_previewViewer->selListSize();
         // check if we have multiple selections.
         if (m_previewViewer->selListSize() > m_selIndex) {
             if ((m_optTlX != nullptr) && (m_optTlY != nullptr) && (m_optBrX != nullptr) && (m_optBrY != nullptr)) {
@@ -1221,12 +1224,16 @@ void KSaneWidgetPrivate::oneFinalScanDone()
                     m_readValsTmr.stop();
                     valReload();
                 }
-                m_updProgressTmr.start();
-                m_scanThread->start();
-                return;
+
+                if (!m_cancelMultiScan) {
+                    m_updProgressTmr.start();
+                    m_scanThread->start();
+                    m_cancelMultiScan = false;
+                    return;
+                }
             }
         }
-        emit(q->scanDone(KSaneWidget::NoError, QStringLiteral("")));
+        emit q->scanDone(KSaneWidget::NoError, QString());
     } else {
         switch (m_scanThread->saneStatus()) {
         case SANE_STATUS_GOOD:
@@ -1234,8 +1241,8 @@ void KSaneWidgetPrivate::oneFinalScanDone()
         case SANE_STATUS_EOF:
             break;
         case SANE_STATUS_NO_DOCS:
-            emit(q->scanDone(KSaneWidget::Information, i18n(sane_strstatus(m_scanThread->saneStatus()))));
-            alertUser(KSaneWidget::Information, i18n(sane_strstatus(m_scanThread->saneStatus())));
+            emit q->scanDone(KSaneWidget::Information, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
+            alertUser(KSaneWidget::Information, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
             break;
         case SANE_STATUS_UNSUPPORTED:
         case SANE_STATUS_IO_ERROR:
@@ -1245,8 +1252,8 @@ void KSaneWidgetPrivate::oneFinalScanDone()
         case SANE_STATUS_COVER_OPEN:
         case SANE_STATUS_DEVICE_BUSY:
         case SANE_STATUS_ACCESS_DENIED:
-            emit(q->scanDone(KSaneWidget::ErrorGeneral, i18n(sane_strstatus(m_scanThread->saneStatus()))));
-            alertUser(KSaneWidget::ErrorGeneral, i18n(sane_strstatus(m_scanThread->saneStatus())));
+            emit q->scanDone(KSaneWidget::ErrorGeneral, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
+            alertUser(KSaneWidget::ErrorGeneral, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
             break;
         }
     }
@@ -1266,7 +1273,7 @@ void KSaneWidgetPrivate::setBusy(bool busy)
         m_activityFrame->hide();
         m_btnFrame->hide();
         m_optionPollTmr.stop();
-        emit(q->scanProgress(0));
+        emit q->scanProgress(0);
     } else {
         m_warmingUp->hide();
         m_activityFrame->hide();
@@ -1274,7 +1281,7 @@ void KSaneWidgetPrivate::setBusy(bool busy)
         if (m_pollList.size() > 0) {
             m_optionPollTmr.start();
         }
-        emit(q->scanProgress(100));
+        emit q->scanProgress(100);
     }
 
     m_optsTabWidget->setDisabled(busy);
@@ -1346,7 +1353,7 @@ void KSaneWidgetPrivate::updateProgress()
     }
 
     m_progressBar->setValue(progress);
-    emit(q->scanProgress(progress));
+    emit q->scanProgress(progress);
 }
 
 void KSaneWidgetPrivate::alertUser(int type, const QString &strStatus)
@@ -1361,7 +1368,7 @@ void KSaneWidgetPrivate::alertUser(int type, const QString &strStatus)
             break;
         }
     } else {
-        emit(q->userMessage(type, strStatus));
+        emit q->userMessage(type, strStatus);
     }
 }
 
@@ -1435,7 +1442,7 @@ void KSaneWidgetPrivate::setPossibleScanSizes()
     m_scanareaPapersize->addItem(i18n("Custom"), customSize);
 
     // Add portrait page sizes
-    for (int sizeCode: m_sizeCodes) {
+    for (int sizeCode: qAsConst(m_sizeCodes)) {
         QSizeF size = QPageSize::size((QPageSize::PageSizeId)sizeCode, QPageSize::Millimeter);
         if (mmToDispUnit(size.width()) > widthInDispUnit) {
             continue;
@@ -1447,7 +1454,7 @@ void KSaneWidgetPrivate::setPossibleScanSizes()
     }
 
     // Add landscape page sizes
-    for (int sizeCode: m_sizeCodes) {
+    for (int sizeCode: qAsConst(m_sizeCodes)) {
         QSizeF size = QPageSize::size((QPageSize::PageSizeId)sizeCode, QPageSize::Millimeter);
         size.transpose();
         if (mmToDispUnit(size.width()) > widthInDispUnit) {
